import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { LoginComponent } from './components/login/login.component';
import { LogoutComponent } from './components/logout/logout.component';
import { IsAuthenticatedDirective } from './directive/is-authenticated.directive';
import { JwtModule, JwtHelperService } from '@auth0/angular-jwt';
import { HttpClientModule } from '@angular/common/http';
import { SecurityConfigurationService } from './services/security-configuration.service';
import { AuthenticationProvider } from './services/authentication.provider';
import { JWTAuthService } from './services/jwt-auth.service';
import { RoleGuard } from './guards/role.guard';
import { AuthGuard } from './guards/auth.guard';



@NgModule({
  declarations: [LoginComponent, LogoutComponent, IsAuthenticatedDirective],
  imports: [
    CommonModule,
    JwtModule,
    HttpClientModule
  ],
  exports: [ LoginComponent, LogoutComponent, IsAuthenticatedDirective]
})
export class AuthenticationModule {
  static forRoot(): ModuleWithProviders {
    return {
      ngModule: AuthenticationModule,
      providers: [
        {
        provide: AuthenticationProvider,
        useClass: JWTAuthService,
        multi: false
        },
        SecurityConfigurationService,
        JwtHelperService,
        RoleGuard,
        AuthGuard
      ]
    };
  }
 }
 export * from './services/jwt-auth.service';
 export * from './services/authentication.provider';
 export * from './services/secure-storage.service';
 export * from './services/secured-storage.provider';
 export * from './services/security-configuration.service';
 export * from './services/secure-storage.factory';
 export * from './models/auth-user.model';
 export * from './models/auth-configuration';
 export * from './models/i-auth-user';
 export * from './guards/auth.guard';
 export * from './guards/role.guard';
 export * from './directive/is-authenticated.directive';
 export * from './components/login/login.component';
 export * from './components/logout/logout.component';
 export * from './mocks/mock-authentication-provider';
 export * from './mocks/mock-config';