import { TestBed, async, inject } from '@angular/core/testing';

import { AuthGuard } from './auth.guard';
import { JwtModule, JwtHelperService } from '@auth0/angular-jwt';
import { AuthenticationProvider } from '../services/authentication.provider';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { AuthProviderMock } from '../mocks/mock-authentication-provider';
import { mockRoutes } from '../../navigation/mocks/routes.mock';
import { EmptyMockComponent } from '../../navigation/mocks/empty-mock.component';

describe('AuthGuard', () => {
  let auth: AuthenticationProvider;
  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, RouterTestingModule.withRoutes(mockRoutes),
        JwtModule.forRoot({
          config: {
            tokenGetter: () => {
              return '';
            }
          }
        })
      ],
      providers: [AuthGuard,
        {
          provide: AuthenticationProvider,
          useClass: AuthProviderMock
        }
        , JwtHelperService ],
        declarations: [EmptyMockComponent]
    });
    auth = TestBed.get(AuthenticationProvider);
  });

  it('should be created', inject([AuthGuard], (service: AuthGuard) => {
    expect(service).toBeTruthy();
  }));
  it('should be activatedf when authenticated', inject([AuthGuard], (service: AuthGuard) => {
    expect(service.canActivate(undefined, undefined)).toBeTruthy();
  }));
  it('should be load if authenticated', inject([AuthGuard], (service: AuthGuard) => {
    expect(service.canLoad(mockRoutes[0], undefined)).toBeTruthy();
    auth.logout();
    expect(service.canLoad(mockRoutes[0], undefined)).toBeFalsy();
  }));
});
