import { TestBed, async, inject } from '@angular/core/testing';

import { RoleGuard } from './role.guard';
import { AuthenticationProvider } from '../services/authentication.provider';
import { AuthProviderMock } from '../mocks/mock-authentication-provider';
import { SecurityConfig, SecurityConfigurationService } from '../services/security-configuration.service';
import { SecurityMockConfig } from '../mocks/mock-config';
import { Component } from '@angular/core';
import { RouterTestingModule } from '@angular/router/testing';
import { ActivatedRouteSnapshot } from '@angular/router';

@Component({
  template: '<router-outlet></router-outlet>'
})
class RoutingComponent { }

@Component({
  template: ''
})
class DummyComponent { }


describe('RoleGuard', () => {
  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [
        RouterTestingModule.withRoutes([
          { path: 'route1', component: DummyComponent, data: { expectedRoles: [] } },
          { path: 'route2', component: DummyComponent, data: { expectedRoles: ['role1'] } },
          { path: 'login', component: DummyComponent, data: { expectedRoles: [] } }
        ])
      ],
      declarations: [DummyComponent, RoutingComponent],
      providers: [RoleGuard,
        { provide: AuthenticationProvider, useClass: AuthProviderMock },
        {
          provide: SecurityConfig,
          useValue: SecurityMockConfig
        },
          SecurityConfigurationService]
    });
  });

  it('should ...', inject([RoleGuard], (guard: RoleGuard) => {
    expect(guard).toBeTruthy();
  }));
  it('should not allow user to overcome the guard for whatever reasons',
  inject([RoleGuard], (guard: RoleGuard) => {
    const fixture = TestBed.createComponent(RoutingComponent);
    const snapshot = new ActivatedRouteSnapshot();
    snapshot.data = { expectedRoles: ['role1'] };
    expect(guard.canActivate(snapshot, undefined)).toBeTruthy();

    snapshot.data = { expectedRoles: ['role2'] };
    expect(() => guard.canActivate(snapshot, undefined)).toThrow();
    // The problem is we can pass a valid ActivatedRouteSnapshot yet - which leaves the other code untested
    // const snapshot2 = new ActivatedRouteSnapshot();
    // snapshot2.data = { expectedRoles: ['Admin'] };

    // expect(guard.canActivate()).toThrowError();
  })
);
});
