import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, UrlTree, Router } from '@angular/router';
import { Observable } from 'rxjs';
import { AuthenticationProvider } from '../services/authentication.provider';
import { SecurityConfigurationService } from '../services/security-configuration.service';

@Injectable({
  providedIn: 'root'
})
export class RoleGuard implements CanActivate {
  private loginUrl = 'login';

  constructor(public auth: AuthenticationProvider, public router: Router, config: SecurityConfigurationService) {
    this.loginUrl = config.getConfiguration().loginUrl;
  }
  canActivate(
    next: ActivatedRouteSnapshot,
    state: RouterStateSnapshot): Observable<boolean | UrlTree> | Promise<boolean | UrlTree> | boolean | UrlTree {
    // this will be passed from the route config
    // on the data property
    // console.log(route);
      if (this.auth.checkRoles(next.data.expectedRoles) && this.auth.isAuthenticated()) {
        return true;
      }
      if( next === undefined ) {
        this.router.navigate([this.loginUrl]);
        // return false;
      }
      if ( 'pathFromRoot' in next ) {
        this.router.navigate([this.loginUrl], { queryParams: { returnUrl: this.generatePath(next.pathFromRoot) } });
      } else {
        this.router.navigate([this.loginUrl]);
      }
      return false;
  }
  private generatePath(allPaths: ActivatedRouteSnapshot[] ): string {
    if (allPaths.length === 0) {
      return '';
    }
    if (allPaths[0]['_routerState'] !== undefined ) {
      return allPaths[0]['_routerState'].url;
    }
    return '';
  }

  
}
