import { Observable } from 'rxjs';
import { AuthUser } from '../models/auth-user.model';

export enum AuthEvents {
    login = 1,
    logout = 2,
    sessionExpired = 3,
    refresh = 4,
    failedLogin = 5
}
// Models
export interface AuthenticationEvent {
    eventType: AuthEvents;
    authData: AuthUser;
    rawData: any;
}
export interface AuthenticationProviderInterface<Type> {
    authenticate(userName: string, password: string): Observable<Type>;
    // Handled by storage provider or Authentication provider I wonder? Too specific!
    // As couple to JWT
    checkRoles(expectedRoles: string[]): boolean;
    getAuthToken(): string;
    getAuthUser(): Type;
    hasRole(role: string): boolean;
    isAuthenticated(): boolean;
    logout(): boolean;
    onAuthChange(): Observable<AuthenticationEvent>;
    refreshSession(): void;
}


/**
 * AuthenticationProvder contract
 * Nb: A recurring pattern to implement the generic type with an abstract class to allow for 
 * Design by contract and Dependency injection 
 * implements {AuthenticationProvider}
 */
export abstract class AuthenticationProvider implements AuthenticationProviderInterface<AuthUser> {

    public abstract authenticate(userName: string, password: string): Observable<AuthUser>;
    public abstract logout(): boolean;
    // Handled by storage provider or Authentication provider I wonder?
    public abstract getAuthToken(): string;
    public abstract checkRoles(expectedRoles: string[]): boolean;
    public abstract hasRole(role: string): boolean;
    public abstract getAuthUser(): AuthUser;
    public abstract isAuthenticated(): boolean;
    public abstract getAuthenticatedState(): Observable<boolean>;
    public abstract refreshSession(): void;
    public abstract onAuthChange(): Observable<AuthenticationEvent>;
}
