import {
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor,
  HttpErrorResponse
} from '@angular/common/http';
import { Observable } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { Injectable, Inject } from '@angular/core';
import { MessageProvider, MessageProviderToken } from '../../messaging/services/message-provider';

 /**
  * @description
  * Inject JWT token for every HttpClient Request 
  * TODO: Use JWTService when we convert Login Component to angular
  * @returns Inject Token in Header
  */
@Injectable({
  providedIn: 'root'
})
export class JwtInterceptorService implements HttpInterceptor {

  constructor( @Inject(MessageProviderToken) private messagesService: MessageProvider) { }
               intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    console.log(' Service is called');
    request = request.clone({
      // setHeaders: {
      //   Authorization: `Bearer ${localStorage.getItem(this.appConstants.jwtKey)}`
      // }
    });
    return next.handle(request)
      .pipe(
        catchError((error: HttpErrorResponse) => {
          let errorMessage = '';
          if (error.error instanceof ErrorEvent) {
            // client-side error
            errorMessage = `Error: ${error.error.message}`;
          } else {
            // server-side error
            errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
          }
          // return message to user via toast
          this.messagesService.sendErrorMessage('http error', errorMessage);

          return next.handle(request);
        })
      );
  }
}
