import { TestBed, inject } from '@angular/core/testing';

import { SecureStorageService} from './secure-storage.service';
import { AuthUser } from '../models/auth-user.model';
import { SecuredStorageProvider } from './secured-storage.provider';
import { StorageModule } from '../../storage/storage.module';
import { StorageConfig } from '../../storage/services/storage-configuration.service';
import { StorageConfigMock } from '../../storage/mocks/config-mock';
import { MockStorageProvider } from '../../storage/mocks/mock-storage-provider';


export class TestModel {
    name: string;
    value: number;
    data: number[];
    constructor(name: string, value: number, data: number[]) {
        this.name = name;
        this.value = value;
        this.data = data;
    }
}

describe('SecureStorageService', () => {
    beforeEach(() => {
        TestBed.configureTestingModule({
            imports: [StorageModule.forRoot()],
            providers: [
                { provide: SecuredStorageProvider, useFactory: () => new SecureStorageService(new MockStorageProvider<AuthUser>() ) },
                { provide: StorageConfig, useValue: StorageConfigMock}
            ]
        });
    });

    it('should be created', inject([SecuredStorageProvider], (service: SecureStorageService) => {
        expect(service).toBeTruthy();
    }));

    it('should store a token', inject([SecuredStorageProvider], (service: SecureStorageService) => {
        service.setDomain('localhost');
        service.setAuthToken('token');
        expect(service.getAuthToken()).toEqual('token');
        service.deleteStorage();
        expect(service.getAuthToken()).toBeUndefined();
    }));

    // It dosn't set User Profile? but why not?
    it('should store an AuthUser', inject([SecuredStorageProvider], (service: SecureStorageService) => {
        service.setDomain('localhost');
        const user: AuthUser = new AuthUser(1);
        service.setAuthUser(user);
        expect(service.getAuthUser().user_profile_id).toEqual(user.user_profile_id);
        service.deleteStorage();
        expect(service.getAuthUser().user_profile_id).toBeUndefined();
    }));
});

