export interface TypeMap { // for mapping from strings to types
    string: string;
    number: number;
    boolean: boolean;
}
type PrimitiveOrConstructor = // 'string' | 'number' | 'boolean' | constructor
  | { new (...args: any[]): any }
  | keyof TypeMap;

type GuardedType<T extends PrimitiveOrConstructor> = T extends { new(...args: any[]): infer U; } ? 
                                                U :
                                                T extends keyof TypeMap ? TypeMap[T] : never;

export function GenericTypeHelper<T>( input: T ): string {
    return typeof input;
}

export function typeGuard<T extends PrimitiveOrConstructor>(o, className: T): boolean {
    const localPrimitiveOrConstructor: PrimitiveOrConstructor = className;
    if (typeof localPrimitiveOrConstructor === 'string') {
        return typeof o === localPrimitiveOrConstructor;
    }
    return o instanceof localPrimitiveOrConstructor;
}
