import { TestBed } from '@angular/core/testing';
import {
  HttpClientTestingModule,
  HttpTestingController,
} from '@angular/common/http/testing';
import { HttpErrorInterceptor } from './http-error-interceptor';
import { HTTP_INTERCEPTORS, HttpClient, HttpErrorResponse } from '@angular/common/http';
import { MessageProvider, MessageProviderToken } from '../../messaging/services/message-provider';
import { MessagingModule } from '../../messaging/messaging.module';
import { MessagingService } from '../../messaging/services/message.service';


describe(`HttpErrorInterceptor`, () => {
  let httpMock: HttpTestingController;
  let messageservice: MessageProvider;
  let httpClient: HttpClient;
  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, MessagingModule],
      providers: [
        {
            provide: HTTP_INTERCEPTORS,
            useClass: HttpErrorInterceptor,
            multi: true,
          },
          {
              provide: MessageProviderToken,
              useClass: MessagingService
        }
      ],
    });

    messageservice = TestBed.get(MessageProviderToken)
    httpClient = TestBed.get(HttpClient);
    httpMock = TestBed.get(HttpTestingController);
  });
  it('should handle an error by sending a message', () => {
    let messageIndex = 0;
    messageservice.getMessages().subscribe( messages => {
        messageIndex ++
        switch(messageIndex) {
            case 1: 
                break;
            case 2:
                expect(messages.length).toEqual(1);
                expect(messages[0].severity).toEqual('error');
                expect(messages[0].detail).toEqual('Http failure response for /test: 0 ');
                break;
        }
    });
    let actualBody;
    httpClient.get('/test').subscribe(
        result => console.log(result),
        (resp: HttpErrorResponse) => {
        actualBody = resp.error;
        // Error shuld be defined in  - maintains errorEvent
        expect(actualBody).toBeDefined();
      });
    const testRequest = httpMock.expectOne('/test');
    testRequest.error(new ErrorEvent('Server Error'));
  });
  it('should handle an error and forward a code', () => {
    let messageIndex = 0;
    messageservice.getMessages().subscribe( messages => {
        messageIndex ++
        switch(messageIndex) {
            case 1: 
                break;
            case 2:
                expect(messages.length).toEqual(1);
                expect(messages[0].severity).toEqual('error');
                expect(messages[0].summary).toEqual('500');
                expect(messages[0].detail).toEqual('Http failure response for /test2: 500 Server Error');
                break;
        }
    });
    let actualBody;
    httpClient.get('/test2').subscribe(
        result => console.log(result),
        (resp: HttpErrorResponse) => {
        actualBody = resp.error;
        // So we maintain the response but handle the error through messaging. 
        // I think I may change this behavior so that network failures do not destabilise the app
        // Would be cool if a network failure could be handled by preserving app state
        expect(resp.status).toEqual(500);
        // Error shuld be defined in 
        expect(actualBody).toEqual({ errorMessage: 'Uh oh!' });
      });
    const testRequest = httpMock.expectOne('/test2');
    testRequest.flush({ errorMessage: 'Uh oh!' }, { status: 500, statusText: 'Server Error' });
    httpMock.verify();
  });
});
