import {
  HttpEvent,
  HttpInterceptor,
  HttpHandler,
  HttpRequest,
  HttpErrorResponse
} from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { Inject } from '@angular/core';
import { MessageProviderToken, MessageProvider } from '../../messaging/services/message-provider';

export class HttpErrorInterceptor implements HttpInterceptor {

  constructor(@Inject(MessageProviderToken) private service: MessageProvider) {

  }
  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    // Could add a retry(1) - not sure it is necessary yet
    return next.handle(request)
      .pipe(
        catchError((error: HttpErrorResponse) => {
          console.warn('Http Error found', error);
          let errorMessage = '';
          if (error.error instanceof ErrorEvent) {
            // client-side error
            errorMessage = `Error: ${error.error.message}`;
          } else {
            // server-side error
            errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
          }
          this.service.sendErrorMessage(error.status.toString(), error.message);
          return throwError(error);
        })
      )
  }
}