import { TestBed } from '@angular/core/testing';
import { MockModel, HelloWorld } from '../mocks/mock-cloneable';
import { handleCloneableResponse, handleCloneableListResponse } from './constructed-response-handler.service';
import { IResponseModel } from '../models/response-model';

const model: HelloWorld = {
  value: 'hello',
  name: 'world'
};

const mockModel = new MockModel();
mockModel.copyConstruct(model);
const responseMockModel: IResponseModel<MockModel> = {
  data: mockModel,
  code: '200',
  messsage: ''
}
const responseMockListModel: IResponseModel<MockModel[]> = {
  data: [mockModel, mockModel, mockModel],
  code: '200',
  messsage: ''
}

describe('ConstructedResponseHandlerService', () => {
  beforeEach(() => TestBed.configureTestingModule({}));

  it('should strip out a mock and construct it', () => {
    const result: MockModel = handleCloneableResponse(responseMockModel, MockModel);
    expect(result).toEqual(mockModel);
    expect(result.getText()).toEqual('hello world');
  });
  it('should strip out a list of mocks and construct them', () => {
    const result: MockModel[] = handleCloneableListResponse(responseMockListModel, MockModel);
    expect(result).toEqual([mockModel, mockModel, mockModel]);
    result.forEach( item =>
      expect(item.getText()).toEqual('hello world'));
  });
});
