import { IResponseModel } from '../models/response-model';
import { ICloneable } from '../../data/models/i-cloneable';

/**
 * Handles cloneable response
 * Access methods as well as values using factory based generic construction
 * Must support empty constructor!  
 * template T 
 * @param type 
 * @param response 
 * @returns cloneable response 
 */
export function handleCloneableResponse<T extends ICloneable<T>>(response: IResponseModel<T>, type: new () => T): T {
  const model = new type();
  model.copyConstruct(response.data);
  return model;
}

/**
 * Access methods as well as values using factory based generic construction
 * Must support empty constructor!
 * Handles cloneable list response
 * template T 
 * @param type 
 * @param response 
 * @returns cloneable list response 
 */
export function handleCloneableListResponse<T extends ICloneable<T>>(response: IResponseModel<T[]>, type: new () => T): T[] {
  return response.data.map(item => {
    const result = new type();
    result.copyConstruct(item);
    return result;
  })
}
