import { Injectable } from '@angular/core';
import { Observable, BehaviorSubject } from 'rxjs';
import { MessageProvider } from '../services/message-provider';
import { IMessage, MessageSeverity } from '../models/i-message';

@Injectable({
    providedIn: 'root'
})
export class MockMessagingService implements MessageProvider {


    private _messages: Map<string, IMessage> = new Map<string, IMessage>();
    constructor() {
        this._messages = new Map<string, IMessage>();
    }
    sendErrorMessage(title: string, message: string): void {
        this._messages.set(title, { severity: MessageSeverity.error, detail: message, summary: title });
    }
    sendInfoMessage(title: string, message: string): void {
        this._messages.set(title, { severity: MessageSeverity.info, detail: message, summary: title });
    }
    sendSuccessMessage(title: string, message: string): void {
        this._messages.set(title, { severity: MessageSeverity.success, detail: message, summary: title });
    }
    sendMessage(severity: string, title: string, message: string): void {
        this._messages.set(title, { severity: MessageSeverity.info, detail: message, summary: title });
    }
    getMessages(): Observable<IMessage[]> {
        const values: IMessage[] = [];
        this._messages.forEach( (value, key) => {
            values.push( value );
        });
        return new BehaviorSubject<IMessage[]>(values);
    }
    sendSimpleMessage(severity: string, title: string, message: string): void {
        throw new Error('Method not implemented.');
    }
    sendDetailedMessage(message: IMessage): void {
        throw new Error('Method not implemented.');
    }
    clear(): void {
        this._messages.clear();
    }
}
