import { TestBed, inject } from '@angular/core/testing';

import { MessagingService } from './message.service';
import { MessageSeverity, IMessage } from '../models/i-message';

describe('MessagingService', () => {
  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [MessagingService]
    });
  });

  it('should be created', inject([MessagingService], (service: MessagingService) => {
    expect(service).toBeTruthy();
  }));
  it('should increment and clear messages', inject([MessagingService], (service: MessagingService) => {
    const messages: IMessage[] = [
      {summary: 'error', detail: 'with an error', severity: MessageSeverity.error },
      {summary: 'warn', detail: 'with a warning', severity: MessageSeverity.warn },
      {summary: 'success', detail: 'with success', severity: MessageSeverity.success },
      {summary: 'info', detail: 'with some infofrmation', severity: MessageSeverity.info}
    ]
    let messageIterator = 0;
    service.getMessages().subscribe( changes => {
      if ( messageIterator === 5) {
        expect(changes.length).toEqual(0);
      } else if ( messageIterator > 0 ) {
        expect(changes.length).toEqual(messageIterator);
        expect(changes[messageIterator - 1].severity).toEqual(messages[messageIterator - 1].severity);
        expect(changes[messageIterator - 1].summary).toEqual(messages[messageIterator - 1].summary);
        expect(changes[messageIterator - 1].detail).toEqual(messages[messageIterator - 1].detail);
      }
      messageIterator++;
    })
    messages.forEach( message => service.sendSimpleMessage(message.severity, message.summary, message.detail));
    service.clear();
  }));
  [
    ['error', 'with an error', MessageSeverity.error ],
    ['warn', 'with a warning', MessageSeverity.warn ],
    ['success', 'with success', MessageSeverity.success ],
    ['info', 'with some infofrmation', MessageSeverity.info ]
  ].forEach( ([summary, detail, severity]) => {
  it('should add a message with ${summary}', inject([MessagingService], (service: MessagingService) => {

    let messageIterator = 0;
    service.getMessages().subscribe( changes => {
      switch ( messageIterator) {
        case 0:
          expect(changes.length).toEqual(0);
          break;
        case 1:
          expect(changes.length).toEqual(1);
          expect(changes[0].severity).toEqual(severity);
          expect(changes[0].summary).toEqual(summary);
          expect(changes[0].detail).toEqual(detail);
          break;
      }
      messageIterator++;
    });
    switch ( severity ) {
      case MessageSeverity.error:
        service.sendErrorMessage(summary, detail);
        break;
      case MessageSeverity.warn:
        service.sendWarningMessage(summary, detail);
        break;
      case MessageSeverity.info:
        service.sendInfoMessage(summary, detail);
        break;
      case MessageSeverity.success:
        service.sendSuccessMessage(summary, detail);
        break;
    }
    }));
  });
});
