import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { IMessage, MessageSeverity } from '../models/i-message';
import { MessageProvider } from './message-provider';
/**
 * Injectable Message Service
 * This may get extended to offer logging options for debugging as well
 * Handy as a simple means for communicating between components
 */
@Injectable()
export class MessagingService implements MessageProvider {
  private _msgs: IMessage[] = [];
  private messages: BehaviorSubject<IMessage[]> = new BehaviorSubject<IMessage[]>([]);
  constructor() { }

  public sendErrorMessage(title: string, message: string) {
    this.sendSimpleMessage(MessageSeverity.error, title, message);
  }
  public sendInfoMessage(title: string, message: string) {
    this.sendSimpleMessage(MessageSeverity.info, title, message);
  }
  public sendSuccessMessage(title: string, message: string) {
    this.sendSimpleMessage(MessageSeverity.success, title, message);
  }
  public sendWarningMessage(title: string, message: string) {
    this.sendSimpleMessage(MessageSeverity.warn, title, message);
  }
  public sendSimpleMessage(severity: MessageSeverity, summary: string, detail: string) {
    // this._msgs = [];
    this._msgs.push( { severity, summary, detail });
    this.messages.next(this._msgs);
  }
  public sendDetailedMessage(message: IMessage): void {
    // this._msgs = [];
    this._msgs.push(message);
    this.messages.next(this._msgs);
  }
  public getMessages(): Observable<IMessage[]> {
    return this.messages.asObservable();
  }
  public clear() {
    this._msgs = [];
    this.messages.next(this._msgs);
  }
}
