import { TestBed } from '@angular/core/testing';
import { Menu } from './menu';
import { IMenuItem } from './i-menu';


const item1: IMenuItem = {
    id:  1,
    title: 'Home',
    path: '',
    icon: 'home',
    roles: ['public'],
    parentId:  0
}
const item2: IMenuItem = {
    id:  2,
    title: 'Module1',
    path: 'module1',
    icon: 'module1',
    roles: ['module1Users'],
    parentId:  1
}
const item3: IMenuItem = {
    id:  3,
    title: 'Module2',
    path: 'module2',
    icon: 'module2',
    roles: ['module2Users'],
    parentId:  1
}

const item4: IMenuItem = {
    id:  4,
    title: 'Module3',
    path: 'module3',
    icon: 'module3',
    roles: ['module1Users', 'module2Users'],
    parentId:  1
}

const item5: IMenuItem = {
    id:  5,
    title: 'Feature',
    path: 'feature1',
    icon: 'feature',
    roles: ['module1Users', 'module2Users'],
    parentId:  4
}

let menuItems: IMenuItem[] = [];
let iterator = 1;
function buildTree(treeDepth: number, rootId: number, itemsPerDepth: number, currentDepth = 0, currentIndex = 0) {
    let items = 0;
    if ( currentDepth === treeDepth ) {
        return;
    }
    while( items <  itemsPerDepth ) {
        // console.log('Adding: ', '/module' + rootId + '/item' + items +'/' + iterator);
            const item: IMenuItem = {
                id:  iterator,
                title: 'item-' + currentIndex,
                path:  iterator + '',
                icon: 'icon',
                children: [],
                roles: ['module1Users', 'admin'],
                parentId:  rootId
            };
            const parentId = iterator;
            iterator ++;
            items ++;
            menuItems.push(item);
            buildTree(treeDepth, parentId, itemsPerDepth, currentDepth + 1, currentIndex * (currentDepth + 1));
    }
    return;
}


describe('MenuClass', () => {
  beforeEach(() => TestBed.configureTestingModule({}));

  it('should be created', () => {
    const menu = new Menu();
    expect(menu).toBeTruthy();
  });
  // NB: We always add a dummy root for our site
  it('should add a root Routes', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    expect(menu.getAllRoutes().length).toEqual(2);
    expect(menu.getAllRoutes()[1]).toEqual(item1);
  });
  // NB: We always add a dummy root for our site
  it('should add a simple tree of Routes', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    expect(menu.getAllRoutes().length).toEqual(5);
    expect(menu.getChildrenForMenuItem(1).length).toEqual(3);
    expect(menu.findItem(2)).toEqual(item2);
    expect(menu.findItem(3)).toEqual(item3);
    expect(menu.findItem(4)).toEqual(item4);
  });
  it('should find by Path', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    expect(menu.findByPath('')).toEqual(item1);
    expect(menu.findByPath('/module1')).toEqual(item2);
    expect(menu.findByPath('/module2')).toEqual(item3);
    expect(menu.findByPath('/module3')).toEqual(item4);
  });
  it('should get depth', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    expect(menu.getDepth(menu.findByPath(''))).toEqual(1);
    expect(menu.getDepth(menu.findByPath('/module1'))).toEqual(2);
    expect(menu.getDepth(menu.findByPath('/module2'))).toEqual(2);
    expect(menu.getDepth(menu.findByPath('/module3'))).toEqual(2);
    menu.addRoute(item5);
    expect(menu.getDepth(menu.findItem(5))).toEqual(3);
  });
  it('should build parent child relationships', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    expect(menu.getParentChildRelationships().get(0)).toEqual([1]);
    expect(menu.getParentChildRelationships().get(1)).toEqual([2, 3, 4]);
    menu.addRoute(item5);
    expect(menu.getParentChildRelationships().get(4)).toEqual([5]);
  });
  it('should get max depth', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    expect(menu.getMaxDepth()).toEqual(1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    expect(menu.getMaxDepth()).toEqual(2);
    menu.addRoute(item5);
    expect(menu.getMaxDepth()).toEqual(3);
  });
  // This uses unshift so root to leaf ordering
  it('should get breadcrumbs irrespective of recursion depth', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    expect(menu.getPathway(menu.findItem(0),4).length).toEqual(1);
    expect(menu.getPathway(menu.findItem(0))[0].id).toEqual(0);

    menu.addRoute(item2);
    expect(menu.getPathway(menu.findItem(1), 4)[0].id).toEqual(0);
    expect(menu.getPathway(menu.findItem(1), 4)[1].id).toEqual(1);
  });
  it('should get breadcrumbs', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    menu.addRoute(item5);
    expect(menu.getPathway(menu.findItem(0)).length).toEqual(1);
    expect(menu.getPathway(menu.findItem(0))[0].id).toEqual(0);

    expect(menu.getPathway(menu.findItem(1)).length).toEqual(2);
    expect(menu.getPathway(menu.findItem(1))[0].id).toEqual(0);
    expect(menu.getPathway(menu.findItem(1))[1].id).toEqual(1);

    expect(menu.getPathway(menu.findItem(2)).length).toEqual(3);
    expect(menu.getPathway(menu.findItem(2))[0].id).toEqual(0);
    expect(menu.getPathway(menu.findItem(2))[1].id).toEqual(1);
    expect(menu.getPathway(menu.findItem(2))[2].id).toEqual(2);

    expect(menu.getPathway(menu.findItem(5), 4).length).toEqual(4);
    expect(menu.getPathway(menu.findItem(5), 4)[0].id).toEqual(0);
    expect(menu.getPathway(menu.findItem(5), 4)[1].id).toEqual(1);
    expect(menu.getPathway(menu.findItem(5), 4)[2].id).toEqual(4);
    expect(menu.getPathway(menu.findItem(5), 4)[3].id).toEqual(5);
  });
  /// Probably this should handled by some history service
  it('should set Active and previous path', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    menu.addRoute(item5);
    menu.setActivePath(item1.url);
    expect(menu.getActiveRoute()).toEqual(item1);
    menu.setActivePath(item2.url);
    expect(menu.getActiveRoute()).toEqual(item2);
    expect(menu.getPreviousRoute()).toEqual(item1);
  });
  it('should get a collection of children', () => {
    const menu = new Menu();
    menu.addRoute(item1);
    menu.addRoute(item2);
    menu.addRoute(item3);
    menu.addRoute(item4);
    menu.addRoute(item5);

    expect(menu.getAllChildren([1,4]).length).toEqual(4);
    expect(menu.getAllChildren([1,4])[0].id).toEqual(2);
    expect(menu.getAllChildren([1,4])[1].id).toEqual(3);
    expect(menu.getAllChildren([1,4])[2].id).toEqual(4);
    expect(menu.getAllChildren([1,4])[3].id).toEqual(5);
  });
  // Need to make sure that out mock is legitimate.
  // it('should generate mock', () => {
  //     // should product fibanacci n series of m ^ n where n = depth, m - items per depth.
  //     // In this cases 4^1 + 4^2 + 4^3 = 84 items
  //     menuItems = [];
  //   buildTree(3, 0, 4);
  //   expect(menuItems.length).toEqual(84);
  //   // ensure ID uniqueness starting
  //   expect(new Set(menuItems.map( x => x.id)).size).toEqual(84);
  //   // ensure max id
  //   expect(Math.max.apply(Math, menuItems.map(  o => o.id ))).toEqual(168);

  //   for( let i = 0; i < menuItems.length; i++ ) {
  //       console.log('length for index: ' + menuItems[i].id,  menuItems.filter(x => x.parentId === menuItems[i].id).length);
  //   }
  // });
  // Putting it all together on a large data set
  it('should build a 3 x 4 tree', () => {
    // should product fibanacci n series of m ^ n where n = depth, m - items per depth.
    // In this cases 4^1 + 4^2 + 4^3 = 84 items
    menuItems = [];
    iterator = 1;
    const menu = new Menu();
    buildTree(3, 0, 4);
    menuItems.forEach( item =>
        menu.addRoute(item));
    // we add 1 for root!
    expect(menu.getAllRoutes().length).toEqual(85);
    let countGroupsOf4 = 0;
    for ( let i = 0; i < menuItems.length; i++ ) {
        expect(menu.getChildrenForMenuItem(menuItems[i].id).length).toEqual(menuItems.filter(x => x.parentId === menuItems[i].id).length);
        if ( menu.getChildrenForMenuItem(menuItems[i].id).length === 4) {
            countGroupsOf4 ++;
        }
    }
    expect(countGroupsOf4).toEqual(20);
    // I can't explain why
    expect(menu.getChildrenForMenuItem(1).length).toEqual(4);
    expect(menu.getChildrenForMenuItem(2).length).toEqual(4);
    expect(menu.getChildrenForMenuItem(3).length).toEqual(0);
    expect(menu.getChildrenForMenuItem(64).length).toEqual(4);
    expect(menu.getChildrenForMenuItem(78).length).toEqual(0);

    expect(menu.getMaxDepth()).toEqual(3);
    expect(menu.getParentChildRelationships().get(0)).toEqual([1, 22, 43, 64]);
    expect(menu.getParentChildRelationships().get(1)).toEqual([2, 7, 12, 17]);
    expect(menu.getDepth(menu.findItem(1))).toEqual(1);
    expect(menu.getParentChildRelationships().get(2)).toEqual([3, 4, 5, 6]);
    expect(menu.getDepth(menu.findItem(2))).toEqual(2);
    expect(menu.getParentChildRelationships().get(3)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(4)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(5)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(6)).toBeUndefined();

    expect(menu.getParentChildRelationships().get(7)).toEqual([8, 9, 10, 11]);
    expect(menu.getDepth(menu.findItem(7))).toEqual(2);
    expect(menu.getParentChildRelationships().get(8)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(9)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(10)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(11)).toBeUndefined();
    expect(menu.getDepth(menu.findItem(11))).toEqual(3);

    expect(menu.getParentChildRelationships().get(12)).toEqual([13,14,15,16]);
    expect(menu.getParentChildRelationships().get(13)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(14)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(15)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(16)).toBeUndefined();
    expect(menu.getDepth(menu.findItem(16))).toEqual(3);

    expect(menu.getParentChildRelationships().get(17)).toEqual([18, 19, 20, 21]);
    expect(menu.getParentChildRelationships().get(18)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(19)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(20)).toBeUndefined();
    expect(menu.getParentChildRelationships().get(21)).toBeUndefined();
    expect(menu.getDepth(menu.findItem(21))).toEqual(3);

    expect(menu.getParentChildRelationships().get(22)).toEqual([23, 28, 33, 38]);
    expect(menu.getDepth(menu.findItem(22))).toEqual(1);
    expect(menu.getDepth(menu.findItem(23))).toEqual(2);

    expect(menu.getParentChildRelationships().get(64)).toEqual([65, 70, 75, 80]);
    expect(menu.getDepth(menu.findItem(64))).toEqual(1);
    expect(menu.getDepth(menu.findItem(70))).toEqual(2);
    expect(menu.getParentChildRelationships().get(80)).toEqual([81, 82, 83, 84]);

    expect(menu.getPathway(menu.findItem(84)).length).toEqual(3);
    expect(menu.getPathway(menu.findItem(84), 2).length).toEqual(2);
    expect(menu.getPathway(menu.findItem(84), 2)[0].id).toEqual(80);
    expect(menu.getPathway(menu.findItem(84), 2)[1].id).toEqual(84);
    expect(menu.findByPath(menu.findItem(84).url).id).toEqual(84);
    });
});
