import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ROUTES, Routes } from '@angular/router';
import { BASE_ROUTE, NavigationService } from './services/navigation.service';
import { NavigationProvider } from './services/navigation-provider';
import { Route } from '@angular/compiler/src/core';

export function provideRoutes(routes: Routes, baseUrl = ''): any {
  return [
    // NB: We are actually overriding Angular's built-in ROUTES Token so we can avoid
    // using multi as we handle it differently
    { provide: ROUTES, multi: true, useValue: routes },
    { provide: BASE_ROUTE, multi: false, useValue: baseUrl },
  ];
}


@NgModule({
  declarations: [],
  imports: [
    CommonModule
  ]
})
export class NavigationModule {
  static forRoot(routes: Routes): ModuleWithProviders {
    return {
      ngModule: NavigationModule,
      providers: [
        provideRoutes(routes),
        {
          provide: NavigationProvider,
          useClass: NavigationService,
          deps: [ROUTES, BASE_ROUTE]
        }]
    };
  }
  static forChild(routes: Route[], baseUrl: string): ModuleWithProviders {
    return {
      ngModule: NavigationModule,
      providers: [
        provideRoutes(routes, baseUrl),
        {
          provide: NavigationProvider,
          useClass: NavigationService,
          deps: [ROUTES, BASE_ROUTE]
        }]
    };
  }
}
export * from './services/navigation-provider';
export * from './services/navigation.service';
export * from './models/i-menu';
export * from './models/menu';
export * from './models/menu-change-event';
export * from './mocks/empty-mock.component'
export * from './mocks/navigation.service.mock';
export * from './mocks/routes.mock';