import { TestBed, inject } from '@angular/core/testing';
import { mockRoutes } from '../mocks/routes.mock';
import { NavigationProvider } from './navigation-provider';
import { NavigationService } from './navigation.service';
import { NavigationModule } from '../navigation.module';


describe('NavigationService', () => {

  beforeEach(() => TestBed.configureTestingModule({
    imports: [
      NavigationModule.forRoot(mockRoutes)
    ],
    declarations: [],
    providers: []
  }));


  // router = TestBed.get(Router);
  it('should be created', inject([NavigationProvider], (service: NavigationService) => {
    expect(service).toBeTruthy();
  }));
  it('should build routes from router config', inject([NavigationProvider], (service: NavigationService) => {
    expect(service.getMenuRoutes().length).toEqual(17);
    expect(service.getMenu().getMaxDepth()).toEqual(3);
    expect(service.getMenu().getChildrenForMenuItem(8).length).toEqual(2);
    expect(service.getMenu().getChildrenForMenuItem(9).length).toEqual(3);
    expect(service.getMenu().getParentChildRelationships().get(9)).toEqual([10,11,12]);
  }));

  it('should filter routes by role', inject([NavigationProvider], (service: NavigationService) => {

    expect(service.getMenuRoutes('private').length).toEqual(7);
    expect(service.getMenuRoutes('public').length).toEqual(7);
    expect(service.getMenuRoutes('user').length).toEqual(4);
    expect(service.getMenuRoutes('user')[0].title).toEqual('feature1');

  }));
  it('should filter routes by role and parent', inject([NavigationProvider], (service: NavigationService) => {

    expect(service.getMenuForRoleAndParent(9, 'private').length).toEqual(2);
    expect(service.getMenuForRoleAndParent(9, 'user').length).toEqual(2);
    expect(service.getMenuForRoleAndParent(9, 'public').length).toEqual(0);
  }));
  it('should set active routes',  inject([NavigationProvider], (service: NavigationService) => {
    service.setActiveRoute('/app2');
    expect(service.getActiveRoute().id).toEqual(8);
    service.setActiveRoute('/app2/child1');
    expect(service.getActiveRoute().id).toEqual(9);
    expect(service.getPreviousRoute().id).toEqual(8);

    service.setActiveRoute('/app2/child1/feature1');
    expect(service.getActiveRoute().id).toEqual(10);
    expect(service.getPreviousRoute().id).toEqual(9);
  }));
  it('should get breadcrumbs for active route',  inject([NavigationProvider], (service: NavigationService) => {
    service.setActiveRoute('/app2') 
    expect(service.getBreadcrumbsForActiveRoute().length).toEqual(2);

    service.setActiveRoute('/app2/child1');
    expect(service.getBreadcrumbsForActiveRoute().length).toEqual(3);

    service.setActiveRoute('/app2/child1/feature1');
    expect(service.getBreadcrumbsForActiveRoute().length).toEqual(3);
    expect(service.getBreadcrumbsForActiveRoute()[0].id).toEqual(8);
    expect(service.getBreadcrumbsForActiveRoute()[1].id).toEqual(9);
    expect(service.getBreadcrumbsForActiveRoute()[2].id).toEqual(10);
  }));
});
