
import { StorageProvider } from '../services/storage.provider';

export class MockStorageProvider<Type> implements StorageProvider<Type> {

    private storage: Map<string, string>;
    private objectStore: Map<string, Type>;
    private domain;
    constructor() {
        this.storage = new Map<string, string>();
        this.objectStore = new Map<string, Type>();
    }
    public setKeyValue(keyName: string, keyValue: string): boolean {
        this.storage.set(keyName, keyValue);
        return true;
    }
    public setObjectValue(keyName: string, keyValue: Type): void {
        this.objectStore.set(keyName, keyValue);
    }
    public getKeyValue(keyName: string): string {
        return this.storage.get(keyName);
    }
    public getKeyObjectValue(keyName: string): Type {
        return this.objectStore.get(keyName);
    }
    public deleteKey(keyName: string): void {
        this.storage.delete(keyName);
    }
    public clearData(): void {
        this.storage.clear(); // = new Map<string, string>();
        this.objectStore.clear();
    }
    public deserialiseJson(json: string, classType: any) {
        return JSON.parse(json, classType);
    }
    public serialiseJson(data: any): string {
        return JSON.stringify(data);
    }
    public hasKey(keyName: string): boolean {
        return this.storage.has(keyName);
    }
    public setDomain(domain: string): void {
        this.domain = domain;
    }


}