import { Injectable, Inject } from '@angular/core';
import { CookieService } from 'ngx-cookie-service';
import { StorageProvider } from './storage.provider';
import { StorageConfigurationService } from './storage-configuration.service';

@Injectable()
export class CookieStorageService extends StorageProvider<any> {

  private path = '/';
  private domain = 'localhost';
  private localhost = 'localhost';
  private expires: Date = new Date();
  // Defaul value not in use private _cookieExpiryHours = 24;
  constructor(private storage: CookieService, configurationProvider: StorageConfigurationService) {
    super();
    // should be like - configuration.getSection<Type>(Key?)?
    // configuration.getSection<SecurityConfiguration>(ModuleConfig.Section)
    // This would then be handled byu the factory
    this.domain = configurationProvider.readConfiguration().cookieUrl;
    this.path = configurationProvider.readConfiguration().cookiePath;
    this.expires.setHours(this.expires.getHours() + configurationProvider.readConfiguration().cookieExpiryHours);
  }
  public setDomain(domain: string) {
    this.domain = domain;
  }
  /**
   * NB: We could add additional security that parses for
   * insecure content like script references
   *
   * @param keyValue
   *  {boolean}
   */
  public setKeyValue(keyName: string, keyValue: string): boolean {
    if (keyValue == null) {
      console.warn('Cookie is Null Value');
      return false;
    }
    // Likely we can't use localhost in domain as per here:
    // https://stackoverflow.com/questions/1134290/cookies-on-localhost-with-explicit-domain
    if (this.domain.includes(this.localhost)) {
      this.storage.set(keyName, keyValue, this.expires);
    } else {
      this.storage.set(keyName, keyValue, this.expires, this.domain);
    }
    // Second check for ensuring cookie is writeable
    return this.storage.get(keyName) === keyValue;
  }

  /**
   * The Object value wrapper for setting client storage dara
   *
   * boolean - true if stored otherwise false
   */
  public setObjectValue(keyName: string, keyValue: any): boolean {
    // note non-strict comparison allows for null or undefined
    if (keyValue == null) {
      return false;
    }
    return this.setKeyValue(keyName, JSON.stringify(keyValue));
  }

  /**
   * Gets the client stored key valye
   *
   * @param  keyName
   * null if not found
   */
  public getKeyValue(keyName: string): string {
    if (this.hasKey(keyName)) {
      return this.storage.get(keyName);
    }
    return null;
  }

  /**
   * Returns the JSON parsed object
   * May be worth consiudering localised deserialisers
   * for each model
   * @param  keyName
   *   null object if not found
   */
  public getKeyObjectValue(keyName: string): any {
    const returnString = this.getKeyValue(keyName);
    try {
      return JSON.parse(returnString);
      
    } catch (e) {
      return null;
    }
  }
  /**
   * Deletes a key
   */
  public deleteKey(keyName: string) {
    if (this.hasKey(keyName)) {
      this.storage.delete(keyName, this.path, this.domain);
    }
    return;
  }
  public clearData() {
    if (this.domain.includes(this.localhost)) {
      this.storage.deleteAll(this.path);
    } else {
      this.storage.deleteAll(this.path, this.domain);
      this.storage.deleteAll('/', this.domain);
      this.storage.deleteAll('/');
      this.storage.deleteAll();
    }

    // this.storage.deleteAll('/', '/');
    // this.storage.deleteAll();
  }
  public hasKey(keyName: string) {
    return this.storage.check(keyName);
  }
}
