import { StorageProvider } from './storage.provider';

export class LocalStorageService extends StorageProvider<any>  {

    constructor() {
        super();
    }
    private domain: string;
    /**
     * A generalised function for setting Key Value pairs in client
     * storage locations. Could add client side cache as well
     * 
     * @param  keyValue
     *  {boolean} true if stored otherwise false
     */
    public setKeyValue(keyName: string, keyValue: string): boolean {
        if (keyValue == null) {
            return false;
        }
        try {
            localStorage.setItem(keyName, keyValue);
            return true;
        } catch (e) {
            return false;
        }
    }

    /**
     * The Object value wrapper for setting client storage dara
     *
     *  boolean - true if stored otherwise false
     */
    public setObjectValue(keyName: string, keyValue: any) {
        if (keyValue == null) {
            return false;
        }
        return this.setKeyValue(keyName, JSON.stringify(keyValue));
    }

    /**
     * Gets the client stored key valye
     *
     *   Empty string if not found
     */
    public getKeyValue(keyName: string): string {
        try {
            return localStorage.getItem(keyName);
        } catch (e) {
            return '';
        }
    }

    /**
     * Returns the JSON parsed object
     * May be worth consiudering localised deserialisers
     * for each model
     *
     * @param  keyName
     *   null object if not found
     */
    public getKeyObjectValue(keyName: string): any {
        const returnString = this.getKeyValue(keyName);
        try {
            return JSON.parse(returnString);
        } catch (e) {
            return null;
        }
    }
    /**
     * Deletes a key
     */
    public deleteKey(keyName: string) {
        try {
            localStorage.removeItem(keyName);
        } catch (e) {
            return;
        }
    }

    /**
     * Deletes all storage
     *
     */
    public clearData() {
        localStorage.clear();
    }

    public hasKey(keyName: string) {
        return localStorage.getItem(keyName) !== null;
    }
    public setDomain(domain: string) {
        this.domain = domain;
    }
}
