import { ISerialisable } from '../../data/models/i-serialisable';


/**
 * A (nearly) purely abstract class because interfaces are somewhat useless
 * in Angular's Dependency Injection strategy. Annoying because I
 * can see a great candidate for making a storageProvider implement
 * an Observable Interface for allowing subscription based models.
 * I would want to inject an Observable variant of my storage Accessor
 * But I can't - meaning I now need to create some class just for
 * Obeservables and build a class Heirarchy around it.
 * template Type
 */
export abstract class StorageProvider<Type> implements ISerialisable  {

    constructor() {}
    /**
     * Sets key value
     * @param keyName identifier
     * @param keyValue value
     *  true if key value
     */
    public abstract setKeyValue(keyName: string, keyValue: string): boolean;

    /**
     * The Object value wrapper for setting client storage data
     *  boolean - true if stored otherwise false
     */
    public abstract setObjectValue(keyName: string, keyValue: Type): void;

    /**
     * Gets the client stored key valye
     *
     * @param keyName
     *  Empty string if not found
     */
    public abstract getKeyValue(keyName: string): string;

    /**
     * Returns the JSON parsed object
     * May be worth consiudering localised deserialisers
     * for each model
     * @param keyName
     *  null object if not found
     */
    public abstract getKeyObjectValue(keyName: string): Type;

    /**
     * Deletes a key
     */
    public abstract deleteKey(keyName: string): void;

    /**
     * Clears data
     */
    public abstract clearData(): void;

    /**
     * Deserialises json
     */
    public deserialiseJson(json: string, classType: any): any {
        return JSON.parse(json);
    }

    /**
     * Serialises json
     */
    public serialiseJson(data: any): string {
        return JSON.stringify(data);
    }

    /**
     * Determines whether key has
     * @param keyName
     *  true if key exists
     */
    public abstract hasKey(keyName: string): boolean;

    /**
     * Sets domain for scoping of storage
     * @param domain without http protocol
     */
    public abstract setDomain(domain: string): void;
}
