// 3rd party decorator
import { ITreeItem } from './i-tree';

export class Category implements ITreeItem {
    children: Category[] = [];
    public id: number;
    public name: string;
    public parentCategoryId: number;
    public depth?: number;

    constructor(id: number, name: string, parentCategoryId: number, children?: Category[],  depth?: number) {
        this.id = id;
        this.name = name;
        this.parentCategoryId = parentCategoryId;
        if( children !== undefined ){
            this.children = children;
        }
        this.depth = depth;
    }
}
export interface CategoryInterface {
    getId(): number;
    getParentCategoryId(): number;
    getName(): string;
    getDescription(): string;
    getSlug(): string;
    getImageUrl(): string;
    getSubCategories(): ITreeItem[];

    setId(id: number): void;
    setParentCategoryId(id: number): void;
    setName(name: string): void;
    setDescription(description: string): void;
    setSlug(slug: string): void;
    setImageUrl(imageUrl: string): void;
    setSubCategories(children: ITreeItem[]): void;
}

export class CategoryModel {
    categoryID: number;

    description?: string;

    imageURL?: string;

    isActive: boolean;

    isPublic: boolean;

    level: number;

    name: string;

    parentCat?: number;

    slug: string;

    sort: number;
}

/**
 * Iselectable - not required by service
 */
export interface ISelectable {
    selected: boolean;
}

