import { Category } from '../models/categories.model';
export abstract class AbstractCategorySelector {
    /**
     * Mainly enforcing type safety. If you can access
     * this and write to it - state won't be broken
     * as init allows for write but you could accidently
     * empty it.
     * 
     * @memberof AbstractCategorySelector
     */
    public abstract getSelectedCatageories(): Category[];

    /**
     * Set the selected Catagories from a list
     *
     * @param selectedCategories
     * @memberof AbstractCategorySelector
     */
    public abstract initSelectedCategories(selectedCategories: Category[]): void;

    /**
     * Select catagory and all parent categories
     * And add them to selected list
     * 
     * @param categoryId
     * @memberof AbstractCategorySelector
     */
    public abstract addSelectedCategory(alias: Category): boolean;

    /**
     * Shallow remove. Will not remove parent categories for child.
     * Could check for orphans - but it starts to seem unintuitve.
     * @param categoryId
     * @memberof AbstractCategorySelector
     */
    public abstract removeSelectedCategory(categoryId: number): boolean;

    /**
     * Get a category from a selected list
     *
     * @param categoryId
     *  {CategoryAlias}
     * @memberof AbstractCategorySelector
     */
    public abstract getSelectedCategory(categoryId: number): Category;

    /**
     * Check if the category has already been selected
     *
     * @param categoryId
     *  {boolean}
     * @memberof AbstractCategorySelector
     */
    public abstract hasSelectedCategory(categoryId: number): boolean;


}
